﻿--[[
onlyHighest = {
	order = -1,
	type = 'toggle',
	name = L["OnlyHighestComboPoints"],
	desc = L["OnlyHighestComboPoints_Desc"],
	get = function()
		return self.db.profile.comboPoints.onlyHighest
	end,
	set = function()
		self.db.profile.comboPoints.onlyHighest = not self.db.profile.comboPoints.onlyHighest
	end,
},
]]--


-------------------------------------------------------------------------------
-- Addon - Combo-Points - Init
-------------------------------------------------------------------------------
-- Version
local MAJOR = "Avion2_ComboPoints"
local MINOR = tonumber(("6"):match("%d+")) or 0

-- Dependencies
if not Avion2Core 				then error(MAJOR .. " requires Avion2.") end
if not LibStub("AceEvent-3.0")	then error(MAJOR .. " requires AceEvent-3.0.") end

-- Create
Avion2_ComboPoints 			= Avion2Core:NewModule(MAJOR, "AceEvent-3.0")
Avion2_ComboPoints.major 	= MAJOR
Avion2_ComboPoints.minor 	= MINOR


-------------------------------------------------------------------------------
-- Addon - Combo-Points - Variables
-------------------------------------------------------------------------------
-- Translation
local L 	= LibStub("AceLocale-3.0"):GetLocale(MAJOR)
local coreL	= LibStub("AceLocale-3.0"):GetLocale(Avion2Core.major)

-- Save settings for new trigger
newSettings = {
	comboPoints	= 0,
	energy 		= 0,
}

-- Local functions
local ComboPointEnergyUpdate


-------------------------------------------------------------------------------
-- Addon - Combo-Points - Register with core
-------------------------------------------------------------------------------
-- Register SavedVariables
function Avion2_ComboPoints:RegisterModuleDB()
	-- Register own saved variables DB
	self.db = Avion2Core.db:RegisterNamespace(Avion2_ComboPoints.major, {
		profile = {
			comboPoints = {},
			lastMinor 	= MINOR,
		},
	})
	
	-- Last combo-points
	self.lastComboPoints = 0
	
	-- Track last energy value
	self.lastUnitPower = 110
	
	-- Updateto to new rev, data converters should go here
	self.db.profile.lastMinor = Avion2_ComboPoints.minor
end


-- (Un)register (un)needed events
function Avion2_ComboPoints:UpdateEventRegistration()
	local enabled = false
	
	-- Scan if combo point is enabled
	if (self:IsEnabled()) then
		for _, valueCP in pairs(self.db.profile.comboPoints) do
			if next(valueCP.energy) then
				enabled = true
				break
			end
		end
	end
		
	-- Combo-Points events
	if (enabled) then
		self:RegisterEvent("UNIT_ENERGY", ComboPointEnergyUpdate)
		self:RegisterEvent("PLAYER_COMBO_POINTS", ComboPointEnergyUpdate)
	else
		self:UnregisterEvent("UNIT_ENERGY")
		self:UnregisterEvent("PLAYER_COMBO_POINTS")
	end
end


-- Update animation (convert animationFunc: string name to function pointer)
function Avion2_ComboPoints:LoadAnimationTables()
	for _, valueCP in pairs(self.db.profile.comboPoints) do
		for _, valueE in pairs(valueCP.energy) do
			Avion2Core:LoadAnimationTables(valueE.animationList)
		end
	end
end


-------------------------------------------------------------------------------
-- Addon - Combo-Points - Options
-------------------------------------------------------------------------------
-- Fetch animationTable from animationID (as suplied by eg. Menu_Reuqest)
function Avion2_ComboPoints:GetAnimationTable(animationID)
	success, _, arg1, arg2, arg3 = self:Deserialize(animationID)
	
	if (success) then
		return self.db.profile.comboPoints[arg1].energy[arg2].animationList[arg3]
	else
		return false
	end
end


-- Build Optionstable
function Avion2_ComboPoints:UpdateOptionTable(args)
	-- Update options when enabled and initialized
	if (self.db and self:IsEnabled()) then
		local serializedInfo
		
		-- Final options
		args[self.major] = {
			type = 'group',
			name = L["ComboPointsEnergy"],
			desc = L["ComboPointsEnergy_Desc"],
			args = {
				new = {
					type = 'group',
					order = 1,
					name = L["NewComboPointsEnergy"],
					desc = L["NewComboPointsEnergy_Desc"],
					args = {
						comboPoints = {
							order = 1,
							type = 'input',
							name = L["NewComboPoints"],
							desc = L["NewComboPoints_Desc"],
							usage = L["NewComboPoints_Usage"],
							--pattern = "(%d)",
							get = function()
								return newSettings.comboPoints
							end,
							set = function(info, val)
								newSettings.comboPoints = tonumber(val)
							end,
						},
						energy = {
							order = 2,
							type = 'input',
							name = L["NewEnergy"],
							desc = L["NewEnergy_Desc"],
							usage = L["NewEnergy_Usage"],
							--pattern = "(%d)",
							get = function()
								return newSettings.energy
							end,
							set = function(info, val)
								newSettings.energy = tonumber(val)
							end,
						},
						addMe = {
							order = 3,
							type = 'execute',
							name = L["AddMe"],
							desc = L["AddMe_Desc"],
							func = function(info)
								if (newSettings.comboPoints and newSettings.comboPoints >= 0 and
									newSettings.energy		and newSettings.energy 		>= 0 and
									newSettings.comboPoints <= 5) then
									-- Submenu for combo-points
									if (not self.db.profile.comboPoints[newSettings.comboPoints]) then
										self.db.profile.comboPoints[newSettings.comboPoints] = {
											energy = {},
											onlyHighest = true,
										}
									end
									
									-- Submenu for energy
									if (not self.db.profile.comboPoints[newSettings.comboPoints].energy[newSettings.energy]) then
										self.db.profile.comboPoints[newSettings.comboPoints].energy[newSettings.energy] = {
											animationList 	= {},
											cancelAnimation	= 0,
										}
										
										Avion2Core:UpdateModuleOptionTable(self)
										self:UpdateEventRegistration()
									end
								end
							end
						},
					},
				},
				highestOnly = {
					type = 'group',
					order = 2,
					name = L["OnlyHighest"],
					desc = L["OnlyHighest_Desc"],
					args = {},
				},
			},
		}
		
		-- Options for each compo-point
		for keyCP, valueCP in pairs(self.db.profile.comboPoints) do
			-- Option that will only trigger highest Threshold for each combo-point
			args[self.major].args.highestOnly.args[string.format("ComboPoint_%d", keyCP)] = {
				type = 'toggle',
				name = string.format(L["ComboPoint"] , keyCP),
				desc = L["OnlyHighest_Desc2"],
				get = function()
					return valueCP.onlyHighest
				end,
				set = function()
					valueCP.onlyHighest = not valueCP.onlyHighest
				end,
			}
			
			-- ... and for each threshold
			for keyE, valueE in pairs(valueCP.energy) do
				args[self.major].args[string.format("ComboPoint_%d-Energy_%d", keyCP, keyE)] = {
					type = 'group',
					name = string.format(L["ComboPointEnergyTitle"], keyCP, keyE),
					desc = L["ComboPointEnergyTitle_Desc"],
					args = { 
						list = {
							order = 1,
							type = 'group',
							name = coreL["AnimationList"],
							desc = coreL["AnimationList_Desc"],
							args = {
								new = Avion2Core:GetDefaultNewAnimationOption(valueE.animationList),
							},
						},
						cancelAnimation = {
							order = 2,
							type = 'select',
							name = coreL["CancelAnimation"],
							desc = coreL["CancelAnimation_Desc"],
							values = {
								[0] = coreL["DontCancel"],
								[1] = coreL["FadeOut"],
								[2] = coreL["Cancel"],
							},
							get = function()
								return valueE.cancelAnimation
							end,
							set = function(info, val)
								valueE.cancelAnimation = val
							end,
						},
						delete = {
							order = 3,
							type = 'execute',
							name = L["DeleteComboPointsEnergy"],
							desc = L["DeleteComboPointsEnergy_Desc"],
							func = function()
								Avion2Core:StopAnimationList(string.format("Avion2_ComboPointEnergy_%d-%d", keyCP, keyE), valueE.animationList, false)
								valueCP.energy[keyE] = nil
								
								if (not next(valueCP.energy)) then
									self.db.profile.comboPoints[keyCP] = nil
								end
								
								Avion2Core:UpdateModuleOptionTable(self)
								self:UpdateEventRegistration()
							end,
						},
					},
				}
				
				-- Add options for each animation
				for animationListID, _ in pairs(valueE.animationList) do
					-- Combine all info to find this animation in module db in ONE string
					serializedInfo = self:Serialize(Avion2_ComboPoints.major, keyCP, keyE, animationListID)
					
					-- Update option table
					args[self.major].args[string.format("ComboPoint_%d-Energy_%d", keyCP, keyE)].args.list.args[string.format("Animation_%d", animationListID)] = Avion2Core:GetDefaultAnimationOption(valueE.animationList, animationListID, serializedInfo)
				end
			end
		end

	-- Disabled, remove options
	else
		args[self.major] = nil
	end
end


-------------------------------------------------------------------------------
-- Addon - Combo-Points - Events
-------------------------------------------------------------------------------
local currentComboPoints, currentUnitPower, highestEnergy


-- Combo-points changed
ComboPointEnergyUpdate = function(eventName, unitID)
	-- Trigger only for player
	if (unitID == "player" and UnitPowerType("player") == 1) then
		-- Save current energy
		currentComboPoints 	= GetComboPoints()
		currentUnitPower 	= UnitPower("player")
		
		-- Iterate overl all combo-points
		for keyCP, valueCP in pairs(Avion2_ComboPoints.db.profile.comboPoints) do
			-- Need to reset this each time!
			highestEnergy = -1
			
			-- For each combo-point, iterate over all energy thresholds
			for keyE, valueE in pairs(valueCP.energy) do
				-- If Combo-Points/Energy combination is found, that fits current values (and wasn't updated in the last push) ...
				if (currentComboPoints 	>= keyCP 	and keyCP 	> Avion2_Energy.lastComboPoints
				and currentUnitPower 	>= keyE 	and keyE	> Avion2_Energy.lastUnitPower) then
					-- Keep track of highest energy threshold (if we also need to keep track of highest combo-points, only update highest energy, when it fits to highest combo-points)
					if  (valueCP.onlyHighest and keyE > highestEnergy) then
						highestEnergy = keyE
					else
						Avion2Core:TriggerAnimationList(string.format("Avion2_ComboPointEnergy_%d-%d", keyCP, keyE), valueE.animationList)
					end
				
				-- Cancel all animation who don't meet requirements anymore
				elseif (valueE.cancelAnimation ~= 0) then
					Avion2Core:StopAnimationList(string.format("Avion2_ComboPointEnergy_%d-%d", keyCP, keyE), valueE.animationList, (valueE.cancelAnimation == 1))
				end
			end
			
			-- Trigger only animations with highest energy threshold
			if (valueCP.onlyHighest and highestEnergy > -1) then
				Avion2Core:TriggerAnimationList(string.format("Avion2_ComboPointEnergy_%d-%d", keyCP, highestEnergy), valueCP.energy[highestEnergy].animationList)
			end
		end
		
		-- Update last combo-points and energy
		Avion2_Energy.lastComboPoints 	= currentComboPoints
		Avion2_Energy.lastUnitPower 	= currentUnitPower
	end
	
	-- !!!Einbauen
	--  Nur animation fur passende (!) combo-punkte/energie kombination mit gruter anzahl kombo-punkte triggern
end
