-------------------------------------------------------------------------------
-- Addon - Core - Init
-------------------------------------------------------------------------------
-- Version
local MAJOR = "Avion2"
local MINOR = tonumber(("6"):match("%d+")) or 0

-- Dependencies
if not LibStub 				  			then error(MAJOR .. " requires LibStub.") 				end
if not LibStub("AceAddon-3.0")  		then error(MAJOR .. " requires AceAddon-3.0.") 			end
if not LibStub("AceDB-3.0")  			then error(MAJOR .. " requires AceDB-3.0.") 			end
if not LibStub("AceDBOptions-3.0") 		then error(MAJOR .. " requires AceDBOptions-3.0.")		end
if not LibStub("AceConfig-3.0")  		then error(MAJOR .. " requires AceConfig-3.0.") 		end
if not LibStub("AceConfigDialog-3.0")  	then error(MAJOR .. " requires AceConfigDialog-3.0.") 	end
if not LibStub("AceSerializer-3.0")		then error(MAJOR .. " requires AceSerializer-3.0.")		end
--if not LibStub("AceEvent-3.0")  		then error(MAJOR .. " requires AceEvent-3.0.") 			end
--if not LibStub("AceHook-3.0")  			then error(MAJOR .. " requires AceHook-3.0.") 			end

-- Create
Avion2Core = LibStub("AceAddon-3.0"):NewAddon(MAJOR, "AceSerializer-3.0")
Avion2Core.major = MAJOR
Avion2Core.minor = MINOR

-- Set default mixins for modules
Avion2Core:SetDefaultModuleLibraries("AceSerializer-3.0")


-------------------------------------------------------------------------------
-- Addon - Core - Variables
-------------------------------------------------------------------------------
-- Ace libs (that don't need to be embeded)
local AceDB				= LibStub("AceDB-3.0")
local AceDBOpts			= LibStub("AceDBOptions-3.0")
local AceConfig			= LibStub("AceConfig-3.0")
local AceConfigDialog	= LibStub("AceConfigDialog-3.0")

-- SharedMedia (if available)
local LibSharedMedia 	= LibStub("LibSharedMedia-3.0", true)

-- Animation core
local LibAvion 			= LibStub("LibAvion-2.0")
local LibAvionOptions 	= LibStub("LibAvionOptions-2.0")

-- Translation
local L 				= LibStub("AceLocale-3.0"):GetLocale(MAJOR)

-- Media DB
Avion2Core.media 		= {
	textureList 	= {},		-- Textures ["name"] = "path"
	soundList 		= {},		-- Sounds ["name"] = "path"
	functionList 	= {},		-- Functions [function] = "name" (name will be saved)
	pointerList 	= {},		-- Functions ["name"] = function (name will be saved)
}

-- Default animation table
local defaultAnimationTable = {
	alpha 		= 1,
	color 			= {
		r		= 1,
		g 		= 1,
		b 		= 1,
	},
	position 		= {
		x 		= 0,
		y 		= 0,
	},
	size 			= {
		width 	= 100,
		height 	= 200,
	},
	rotation 		= 0,
	isAnimated 		= {
		color 		= true,
		position 	= true,
		scale 		= true,
		rotation 	= true,
	},
	animation 		= {
		color 		= {
			r 	= 0,
			g 	= 0,
			b 	= 1,
		}, 
		position 	= {
			x 	= 0,
			y 	= 0,
		}, 
		scale 		= 1,
		rotation 	= 0,
	},
	animationFunc	= {
		alpha 			= nil,
		color 			= nil,
		position 		= nil,
		scale 			= nil,
		rotation 		= nil,
	},
	animationFuncNames	= {
		alpha 			= "Linear",
		color 			= "Linear",
		position 		= "Linear",
		scale 			= "Linear",
		rotation 		= "Linear",
	},
	addRandom 		= {
		alpha 			= 0,
		color 			= 0,
		position 		= 0,
		scale 			= 0,
		rotation 		= 0,
	},
	fadeMode 		= 1,
	animationTime 	= 5,
	fadeTime 		= 2,
	waitTime 		= 0,
	showWhileWait 	= true,
	textureFileName	= "Flames",
	textureFile 	= "",
	soundFileName	= "Boing",
	soundFile 		= "",
	mirror 		= {
		x 		= false,
		y 		= false,
	},
	noEnd 		= false,
}


-------------------------------------------------------------------------------
-- Library - Utility functions
-------------------------------------------------------------------------------
-- Create a deep copy of table (Keep same metatable)
local function deepcopy(object)
	local lookup_table = {}
	
	local function _copy(object)
		if type(object) ~= "table" then
			return object
		elseif lookup_table[object] then
			return lookup_table[object]
		end
		
		local new_table = {}
		lookup_table[object] = new_table
		
		for index, value in pairs(object) do
			new_table[_copy(index)] = _copy(value)
		end
		
		return setmetatable(new_table, getmetatable(object))
	end
	
	return _copy(object)
end


-------------------------------------------------------------------------------
-- Addon - Core - Module prototype
-------------------------------------------------------------------------------
-- Prototype table
Avion2Core.modulePrototype = {}

-- Say Hello to core
function Avion2Core.modulePrototype:OnInitialize()
	-- If module loads for the first time (== nil) enable it (=true)
	if (Avion2Core.db.profile.modules[self.major] == nil) then
		Avion2Core.db.profile.modules[self.major] = true
	end
	
	-- Register module save DB
	self:RegisterModuleDB()
	
	-- Load animation tables from db (convert them)
	self:LoadAnimationTables()
end


-- Enable event-listerning when enabling module
function Avion2Core.modulePrototype:OnEnable()
	-- Disable again if not enabled by Avion2Core
	if (not Avion2Core.db.profile.modules[self.major]) then
		self:Disable()
	else
		-- Register events to listen to
		self:UpdateEventRegistration()
		
		-- Add/Update option table
		Avion2Core:UpdateModuleOptionTable(self)
	end
end


-- Disable event-listerning when disabling module
function Avion2Core.modulePrototype:OnDisable()
	-- Unregister events
	self:UpdateEventRegistration()
	
	-- Update option table
	Avion2Core:UpdateModuleOptionTable(self)
end


-- Set as default prototype
Avion2Core:SetDefaultModulePrototype(Avion2Core.modulePrototype)


-------------------------------------------------------------------------------
-- Addon - Core - Common
-------------------------------------------------------------------------------
-- Database with saved variables
function Avion2Core:OnInitialize()
	-- Register database
	--self.db = AceDB:New(MAJOR.."DB", defaultDB, "profile")
	self.db = AceDB:New(MAJOR.."DB")
	self.db:RegisterDefaults({
		profile = {
			modules		= {},
			lastMinor 	= MINOR,
		},
	})
	
	-- Register events that are fired on profile change
	self.db.RegisterCallback(self, "OnNewProfile", 		"ProfileChanges")
	self.db.RegisterCallback(self, "OnProfileReset", 	"ProfileChanges")
	self.db.RegisterCallback(self, "OnProfileChanged", 	"ProfileChanges")
	self.db.RegisterCallback(self, "OnProfileCopied", 	"ProfileChanges")
	
	-- Load sounds from SharedMedia
	if (LibSharedMedia) then
		for _, name in pairs(LibSharedMedia:List("sound") or {}) do
			self:AddSound(name, LibSharedMedia:Fetch("sound", name))
		end
	end
	
	-- Default/Main options
	self.options = {
		type = 'group',
		name = L["AddonName"],
		desc = L["AddonName_Desc"],
		icon = [[Interface\AddOns\Avion\Icon.tga]],
		args = {
			modules = {
				type = 'group',
				name = L["Modules"],
				desc = L["Modules_Desc"],
				args = {},
			},
		},
	}
	
	-- Add profile handling for AceConfig-3.0
	self.options.args.profile = AceDBOpts:GetOptionsTable(self.db)
	
	-- Register options to AceConfig-3.0
	AceConfig:RegisterOptionsTable(MAJOR, self.options)
	
	AceConfigDialog:AddToBlizOptions(MAJOR, MAJOR)
	
	-- Events that need to be watched regardless of own state
	if (LibSharedMedia) then
		LibSharedMedia.RegisterCallback(self, "LibSharedMedia_Registered", "LibSharedMedia_Registered")
	end
end


-- Enable everything
function Avion2Core:OnEnable()
	-- If any Module available, register its events
	for moduleName, module in self:IterateModules() do
		-- Test if it should be enabled or not
		if (self.db.profile.modules[moduleName]) then
			if (not module:IsEnabled()) then
				module:Enable()
			end
			
		-- Test if it is (for some reason) allready enabled (disable since it shouldn't be!)
		elseif (module:IsEnabled()) then
			module:Disable()
		end
	end
	
	-- Register menu request (by guideFrame)
	LibAvionOptions.RegisterCallback(self, "LibAvion_MenuRequest", "LibAvion_MenuRequest")
	
	-- Update optiontable once	
	self:UpdateOptionTable()
end


-- Disable everything
function Avion2Core:OnDisable()
	-- Stop all animations
	LibAvion:StopAllAnimations(false)
	
	-- Unregister all events
	for _, module in self:IterateModules() do
		if (module:IsEnabled()) then
			module:Disable()
		end
	end
	
	-- Unregister menu event
	LibAvionOptions.UnregisterCallback(self, "LibAvion_MenuRequest", "LibAvion_MenuRequest")
end


-------------------------------------------------------------------------------
-- Addon - Core - Options
-------------------------------------------------------------------------------
-- Update complete option table
function Avion2Core:UpdateOptionTable()
	local moduleL
	for _, module in self:IterateModules() do
		moduleL = LibStub("AceLocale-3.0"):GetLocale(module.major, true)
		
		-- Options to enable/disable module
		self.options.args.modules.args[module.major] = {
			type = 'toggle',
			name = moduleL["ModuleName"],
			desc = L["EnableModule_Desc"],
			get = function()
				return (self.db.profile.modules[module.major] == true)
			end,
			set = function()
				-- Change enabled status in core
				self.db.profile.modules[module.major] = not self.db.profile.modules[module.major]
				
				-- Change enabled status of module itsself
				if (self.db.profile.modules[module.major]) then
					module:Enable()
				else
					module:Disable()
				end
			end,
		}
	end
			
	-- Update module options
	self:UpdateAllModulesOptionTable()
	
	--LibStub("AceConfigRegistry-3.0"):NotifyChange(MAJOR)
end


-- Update modules option table
function Avion2Core:UpdateModuleOptionTable(module)
	-- Make sure module exists
	if (self:GetModule(module.major, true)) then
		-- Feed option table into the core
		module:UpdateOptionTable(self.options.args)
	end
end


-- Update option table for all modules
function Avion2Core:UpdateAllModulesOptionTable()
	for _, module in self:IterateModules() do
		self:UpdateModuleOptionTable(module)
	end
end


-- User changed profile,
function Avion2Core:ProfileChanges()
	-- Stop all animations
	LibAvion:StopAllAnimations(false)
	
	-- Update option table
	self:UpdateOptionTable()
end


-- Menu was requested by a GuideFrame
function Avion2Core:LibAvion_MenuRequest(animationID)
	--[[
	-- Make sure animationID type is correct
	if (type(animationID) ~= "string") then
		return false
	end
	
	-- Fetch addon name
	success, moduleName = self:Deserialize(animationID)
	
	-- Test if its a module of Avion2
	if (not success or not moduleName:match("Avion2_(%a+)")) then
		return false 
	end
	
	-- Test if module really exists and fetch it
	local module = self:GetModule(moduleName, true)
	if (not module) then
		return false
	end
	
	-- Fetch animationTable
	local animationTable = module:GetAnimationTable(animationID)
	
	-- Create option table for this animation
	local options = LibAvionOptions:BuildAnimationOptions("ACE3.0", animationID, animationTable, self.media.textureList, self.media.soundList, self.media.functionList)
	]]--
	
	-- (Wait until " AceConfigDropdown-3.0" gets finished)
	LibStub("AceConfigDialog-3.0"):Open(MAJOR)
	
	return true
end


-- Create default options for creating new animation
function Avion2Core:GetDefaultNewAnimationOption(db)
	return {
		order = 1,
		type = 'execute',
		name = L["NewAnimation"],
		desc = L["NewAnimation_Desc"],
		func = function()
			table.insert(db, deepcopy(defaultAnimationTable))
			self:LoadAnimationTables(db)
			
			self:UpdateOptionTable()
		end
	}
end


-- Create default options for animation
function Avion2Core:GetDefaultAnimationOption(db, animationListID, animationID)
	-- Default options as supplied by LibAvionOptions-2.0
	local options = LibAvionOptions:BuildAnimationOptions("ACE3.0", animationID, db[animationListID], self.media.textureList, self.media.soundList, self.media.functionList)
	
	-- Test animation by triggering it
	options["testAnimation"] = {
		order = -2,
		type = 'execute',
		name = L["RunAnimation"],
		desc = L["RunAnimation_Desc"],
		func = function()
			LibAvion:StopAnimation(animationID, false)
			LibAvion:TriggerAnimation(animationID, db[animationListID])
		end
	}
	
	-- Delete options
	options["delete"] = {
		order = -1,
		type = 'execute',
		name = L["DeleteAnimation"],
		desc = L["DeleteAnimation_Desc"],
		func = function()
			LibAvion:StopAllAnimations(false)
			LibAvionOptions:HideGuideFrame(animationID)
			
			table.remove(db, animationListID)
			
			self:UpdateOptionTable()
		end
	}
	
	return {
		type = 'group',
		name = string.format(L["AnimtionNo"], animationListID),
		desc = L["AnimationNo_Desc"],
		args = options
	}
end


-------------------------------------------------------------------------------
-- Addon - Core - Custom content
-------------------------------------------------------------------------------
-- Add new texture to DB
function Avion2Core:AddTexture(name, path)
	-- Terminate if allready something with that name
	if (self.media.textureList[path]) then
		return false
	end
	
	-- Terminate if already entry with that value
	for _, _name in pairs(self.media.textureList) do
		if (_name == name) then
			return false
		end
	end
	
	-- Add me
	self.media.textureList[path] = name
	
	-- Update options
	self:UpdateOptionTable()
	
	return true
end


-- Add new sound to DB
function Avion2Core:AddSound(name, path)
	-- Terminate if allready something with that name
	if (self.media.soundList[path]) then
		return false
	end
	
	-- Terminate if already entry with that value
	for _, _name in pairs(self.media.soundList) do
		if (_name == name) then
			return false
		end
	end
	
	-- Add me
	self.media.soundList[path] = name
	
	-- Update options
	self:UpdateOptionTable()
	
	return true
end


-- Add new function to DB
function Avion2Core:AddFunction(name, body)
	-- Terminate if allready something with that name
	if (self.media.functionList[name]) then
		return false
	end
	
	local func = assert(loadstring([[
		return function (param, rnd) 
			]] .. body ..[[
		end
	]]))
	local result = func()
	
	-- Add me
	self.media.functionList[result] = name
	self.media.pointerList[name] = result
	
	-- New function added, try to update module animations
	for _, module in self:IterateModules() do
		module:LoadAnimationTables()
	end
	
	-- Update options
	self:UpdateOptionTable()
	
	return true
end


-------------------------------------------------------------------------------
-- Addon - Core - Misc
-------------------------------------------------------------------------------
-- Trigger MANY animations at once
function Avion2Core:TriggerAnimationList(triggerID, animationList)
	-- Only trigger animations if enabled
	if (not self:IsEnabled() or animationList == nil) then
		return false
	end
	
	-- Trigger each animation
	for ID, animationTable in pairs(animationList) do
		LibAvion:TriggerAnimation(string.format("%s%d", triggerID, ID), animationTable)
	end
end


-- Stop MANY animations at once
function Avion2Core:StopAnimationList(triggerID, animationList, fadeOut)
	-- Only trigger animations if enabled
	if (animationList == nil) then
		return false
	end
		
	-- Trigger each animation
	for animationID, _ in pairs(animationList) do
		LibAvion:StopAnimation(string.format("%s%d", triggerID, animationID), fadeOut)
	end
end


-- Update animation (convert animationFunc: string name to function pointer)
function Avion2Core:LoadAnimationTables(animationList)
	for _, animationTable in pairs(animationList) do
		LibAvionOptions:ConvertAnimationTable(animationTable, self.media.textureList, self.media.soundList, self.media.functionList)
	end
end


-- If new media is added, also add it to own list
function Avion2Core:LibSharedMedia_Registered(name, mediaType, key)
	if (mediaType == "sound") then
		self:AddSound(key, SharedMedia:Fetch(mediatype, key))
	end
end
